%GaussAR - Probability distribution class, representing
%a Gaussian autoregressive process
%
%Gustav Eje Henter 2012-11-16

classdef GaussAR < GaussD
    properties (GetAccess=public,SetAccess=public)
        Order = 0;
        SummMean = 0; % Mean parameters of summarizers
        ARCoeff = 0;
    end
    methods (Access=public) 
        function pD = GaussAR(varargin)
            switch nargin
                case 0;%default scalar normalized Gaussian random variable
                case 1
                    arg1=varargin{1};
                    if isa(arg1,'GaussAR')
                        pD = arg1;%copy it without transformation
                    else
                        pD = init(pD,arg1);
                        %error('Argument must be a GaussAR object');
                    end;
                otherwise
                     pD=setproperties(pD,varargin{:});%and set all its properties
             end;%switch
        end
        %
        %***** Methods for General Use:
        [X,history] = rand(pD,nData,history);
        [X,history] = mode(pD,nData,history);
        logP = logprob(pD,x);
        %
        %***** Training Methods:
        %
        [pD,iOK] = init(pD,xs,lxs);
        aState = adaptStart(pD);
        aState = adaptAccum(pD,aState,xs,gammas);
        pD = adaptSet(pD,aState);
    end
    methods%***** set/get methods
        %function v=get.Variance(pD) % Not quite correct
        %    v=pD.StDev.^2;
        %end
    end
    methods (Access='private')
        sm = getSummMat(pD);
        [X,history] = driveAR(pD,nData,history,R);
        function [pD,OK]=setproperties(pD,varargin)
        %[pD,OK]=setproperties(pD,propName,value,...) sets value of property propName
        %for a single GaussD object or array of GaussD objects
        %(Mainly for backward compatibility)
        %
        %Input:
        %pD=    the object or object array
        %propName, value=   property name, property value
        %If value is a numeric array,
        %       ALL elements in the object array pD are assigned the new property value.
        %If value is a cell array, it must have the same size as the GaussD array,
        %       and then each object is assigned with the corresponding value cell.
        %
        %Several (propName,value) argument pairs may follow.
        %
        %Note:      If properties StDev or Variance are set,
        %           property AllowCorr is NOT changed, only
        %           the covariance eigenvalues.
        %           Thus, the main axes of a correlated distribution
        %           are NOT changed.
        %
        %           If property Covariance is set, 
        %           a full covariance matrix is allowed,
        %           including non-zero correlations, i.e.
        %           property AllowCorr is set to TRUE.
        %
        %Result:
        %pD=    modified object(s)
        %OK=    TRUE, if properties are acceptable in all pD elements
        %
        %Arne Leijon 2007-02-01 tested
            sizObj=size(pD);%object array size
            nObj=prod(sizObj);%number of objects in array
            property_argin = varargin;
            while length(property_argin) >= 2,
                propName = property_argin{1};
                v = property_argin{2};
                property_argin = property_argin(3:end);
                if iscell(v)
                    if any(sizObj~=size(v))
                        error([propName,' cell array has incompatible size']);end;
                end;
                switch propName
%                     case 'DataSize'
%                         warning('DataSize is specified only by Mean property');
                    case 'Mean'
                        if iscell(v)
                            for i=1:nObj
                                pD(i).Mean=v{i}(:);%each object gets separate value
                            end;
                        else
                            for i=1:nObj
                                pD(i).Mean=v(:);%all objects get same value
                            end;
                        end;
                    case 'StDev'
                        if iscell(v)
                            for i=1:nObj
                                pD(i).StDev=abs(v{i}(:));%each object gets separate value
                            end;
                        else
                            for i=1:nObj
                                pD(i).StDev=abs(v(:));%all objects get same value
                            end;
                        end;
                    case 'Variance'
                        if iscell(v)
                            for i=1:nObj
                                pD(i).StDev=sqrt(abs(v{i}(:)));%each object gets separate value
                            end;
                        else
                            for i=1:nObj
                                pD(i).StDev=sqrt(abs(v(:)));%all objects get same value
                            end;
                        end;
                    case 'Covariance'
                        if iscell(v)
                            for i=1:nObj
                                pD(i)=setCov(pD(i),v{i});%each object gets separate value
                            end;
                        else
                            for i=1:nObj
                                pD(i)=setCov(pD(i),v);%each object gets same value
                            end;
                        end;
                    case 'Order'
                        if iscell(v)
                            for i=1:nObj
                                pD(i).Order=v{i};%each object gets separate value
                            end;
                        else
                            for i=1:nObj
                                pD(i).Order=v;%each object gets same value
                            end;
                        end;
                    case 'SummMean'
                        if iscell(v)
                            for i=1:nObj
                                pD(i).SummMean=v{i};%each object gets separate value
                            end;
                        else
                            for i=1:nObj
                                pD(i).SummMean=v;%each object gets same value
                            end;
                        end;
                    case 'ARCoeff'
                        if iscell(v)
                            for i=1:nObj
                                pD(i).ARCoeff=v{i};%each object gets separate value
                            end;
                        else
                            for i=1:nObj
                                pD(i).ARCoeff=v;%each object gets same value
                            end;
                        end;
                    case 'UserData'
                        if iscell(v)
                            for i=1:nObj
                                pD(i).UserData=v{i};%each object gets separate value
                            end;
                        else
                            for i=1:nObj
                                pD(i).UserData=v;%all objects get same value
                            end;
                        end;
                    case 'AllowCorr'
                        if all(size(v)==1)%single value, each object gets same value
                            if v==1%TRUE, set full covariance
                                for i=1:nObj
                                    if ~allowsCorr(pD(i))%use StDev for full cov matrix
                                        pD(i).CovEigen=eye(length(pD(i).Mean));%id matrix
                                    end;%else already set, no change
                                end;
                            else%force diagonal
                                for i=1:nObj
                                    if allowsCorr(pD(i))%set to NOT allow Corr
                                        pD(i).CovEigen=1;%force diagonal covariance matrix;
                                    end;%else no change
                                end;
                            end;
                        else
                            if any(sizObj~=size(v))
                                error([propName,' data has incompatible size']);end;
                            for i=1:nObj
                                if v(i)==1%TRUE, set full covariance
                                    if ~allowsCorr(pD(i))%set it
                                        pD(i).CovEigen=eye(length(pD(i).Mean));%id matrix
                                    end;%else already set, no change
                                else%force diagonal
                                    if allowsCorr(pD(i))%set to NOT allow Corr
                                        pD(i).CovEigen=1;%force diagonal covariance matrix;
                                    end;%else no change
                                end;                       
                            end;
                        end;
                    otherwise
                        warning('GaussD:CannotSet',['Cannot set property ',propName,' of this object']);
                end;%switch
            end;%while
            %OK=propertiesOK(pD);%check all properties    
        end
        %function OK=propertiesOK(pD)%check consistency of properties
        %    nObj=numel(pD);
        %    OK=true;
        %    for i=1:nObj
        %        if isempty(pD(i).Mean)
        %            OK=false;
        %            warning('GaussD:propCheck','No Mean value');return;
        %        end;%cannot check further
        %        if isempty(pD(i).StDev)
        %            OK=false;
        %            warning('GaussD:propCheck','No StDev value');return;
        %        end;%cannot check further
        %        if length(pD(i).StDev) ~= length(pD(i).Mean)
        %            OK=false;
        %            warning('GaussD:propCheck','Mean and Covariance have incompatible size');
        %        end;
        %    end; 
        %end
    end
end
